/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.task.field;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.Icon;
import javax.swing.SwingUtilities;
import nl.cloudfarming.client.field.model.ActivityField;
import nl.cloudfarming.client.field.model.TreatmentZone;
import nl.cloudfarming.client.geoviewer.BaseLayer;
import nl.cloudfarming.client.geoviewer.SimpleLayer;
import nl.cloudfarming.client.model.EntitySavable;
import nl.cloudfarming.client.task.field.tree.FieldTaskNode;
import org.netbeans.api.actions.Savable;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.MIMEResolver;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObject.Registration;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.MultiDataObject;
import org.openide.loaders.MultiFileLoader;
import org.openide.nodes.Node;
import org.openide.util.Exceptions;
import org.openide.util.ImageUtilities;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.lookup.AbstractLookup;
import org.openide.util.lookup.InstanceContent;
import org.openide.util.lookup.ProxyLookup;

@MIMEResolver.Registration(displayName = "#FieldTaskDataObject displayName", resource = "FieldTaskResolver.xml")
@Registration(displayName = "#FieldTaskDataObject displayName", iconBase = "nl/cloudfarming/client/task/field/to_do_list_checked_1.png", mimeType = "application/x-agrosense-field-task")
@NbBundle.Messages("FieldTaskDataObject displayName=Field Task")
public class FieldTaskDataObject extends MultiDataObject implements PropertyChangeListener {

    private static final Logger LOGGER = Logger.getLogger(FieldTaskDataObject.class.getCanonicalName());
    private InstanceContent ic;
    private AbstractLookup intern;
    private FieldTask fieldTask;
    private FieldTaskManager fieldTaskManager;
//    private final CountDownLatch latch = new CountDownLatch(1);

    public FieldTaskDataObject(FileObject pf, MultiFileLoader loader) throws DataObjectExistsException, IOException {
        super(pf, loader);
        registerEditor(FieldTask.MIME_TYPE, true);

        ic = new InstanceContent();
        intern = new AbstractLookup(this.ic);

        fieldTaskManager = Lookup.getDefault().lookup(FieldTaskManager.class);
        assert fieldTaskManager != null : "No FieldTaskManager class found in the default lookup, unable to load the DataObject";

        //
        // file IO to background thread
        //
        if (SwingUtilities.isEventDispatchThread()){
            LOGGER.warning("FIXME: FieldTaskDataObject performs file IO tasks and should not be initialized on the EDT");
        }
//        SwingUtilities.invokeLater(new Runnable() {
//            @Override
//            public void run() {
                fieldTask = fieldTaskManager.loadTask(getPrimaryFile());
                assert fieldTask != null;
                assert fieldTask.getLookup() != null : "FieldTask lookup is null, not properly deserialized";
                LOGGER.log(Level.FINE, "FieldTaskDataObject loaded {0}", System.identityHashCode(fieldTask));
                ic.add(fieldTask);
                ic.add(new BaseLayer(createPalette(fieldTask), fieldTask.getName()));
                fieldTask.addPropertyChangeListener(FieldTask.PROP_TREATMENTZONE_VALUE, FieldTaskDataObject.this);
                fieldTask.addPropertyChangeListener(FieldTask.PROP_WORK_WIDTH, FieldTaskDataObject.this);
//                latch.countDown();
//            }
//        });

    }

    @Override
    protected int associateLookup() {
        return 1;
    }

    @Override
    protected Node createNodeDelegate() {
//        if (latch.getCount() > 0) {
//            LOGGER.warning("FIXME:createNodeDelegate called directly after construction, initialization not finished!");
//        }
//        boolean finished = false;
//        try {
//            finished = latch.await(5, TimeUnit.SECONDS);
//        } catch (InterruptedException ex) {
//            Exceptions.printStackTrace(ex);
//        }
//        if (finished) {
            return new FieldTaskNode(super.createNodeDelegate(), fieldTask);
//        } else {
//            throw new IllegalStateException();
//        }
    }

    @Override
    public Lookup getLookup() {
//        if (latch.getCount() > 0) {
//            LOGGER.warning("FIXME:getLookup called directly after construction, initialization not finished!");
//        }
//        boolean finished = false;
//        try {
//            finished = latch.await(5, TimeUnit.SECONDS);
//        } catch (InterruptedException ex) {
//            Exceptions.printStackTrace(ex);
//        }
//        if (finished) {
            return new ProxyLookup(super.getLookup(), fieldTask.getLookup(), intern);
//        } else {
//            throw new IllegalStateException();
//        }

    }

    @Override
    public void propertyChange(PropertyChangeEvent evt) {
        Savable existingSavable = getLookup().lookup(Savable.class);
        if (existingSavable == null) {
            ic.add(new FieldTaskSavable());
        }
    }

    @Override
    protected void handleDelete() throws IOException {
        super.handleDelete();
        FieldTaskSavable existingSavable = getLookup().lookup(FieldTaskSavable.class);
        if (existingSavable != null) {
            existingSavable.destroy();
        }
    }
    private static final Icon ICON = ImageUtilities.loadImageIcon("nl/cloudfarming/client/task/field/to_do_list_checked_1.png", true);

    private class FieldTaskSavable extends EntitySavable {

        public FieldTaskSavable() {
            super(fieldTask, ICON);
            register();
        }

        @Override
        protected String findDisplayName() {
            return fieldTask.getName();
        }

        @Override
        protected void handleSave() throws IOException {
            getDataObject().fieldTaskManager.save(fieldTask);
            destroy();
        }

        @Override
        public void destroy() {
            getDataObject().ic.remove(FieldTaskSavable.this);
            unregister();
        }

        @Override
        public void dispose() {
            getDataObject().dispose();
        }

        @Override
        public boolean equals(Object obj) {
            if (obj instanceof FieldTaskSavable) {
                FieldTaskSavable savable = (FieldTaskSavable) obj;
                return getDataObject() == savable.getDataObject();
            }
            return false;
        }

        private FieldTaskDataObject getDataObject() {
            return FieldTaskDataObject.this;
        }

        @Override
        public int hashCode() {
            return getDataObject().hashCode();
        }
    }

    private static FieldTaskPalette createPalette(FieldTask task) {
        ActivityField activityField = task.getLookup().lookup(ActivityField.class);
        assert activityField != null : "activity field needed in the lookup of the task";
        Double minValue = getPaletteMinValue(activityField.getTreatmentZones());
        Double maxValue = getPaletteMaxValue(activityField.getTreatmentZones());
        FieldTaskPalette fieldTaskPalette = new FieldTaskPalette(minValue, maxValue, task.getUom());
        return fieldTaskPalette;
    }

    private static Double getPaletteMinValue(List<TreatmentZone> treatmentZones) {
        Double minValue = null;
        for (TreatmentZone treatmentZone : treatmentZones) {
            double value = treatmentZone.getValue();
            if (minValue == null || value < minValue) {
                minValue = value;
            }
        }
        return minValue;
    }

    private static Double getPaletteMaxValue(List<TreatmentZone> treatmentZones) {
        Double maxValue = null;
        for (TreatmentZone treatmentZone : treatmentZones) {
            double value = treatmentZone.getValue();
            if (maxValue == null || value > maxValue) {
                maxValue = value;
            }
        }
        return maxValue;
    }
}
