/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.task.field;

import java.awt.Color;
import java.io.Serializable;
import nl.cloudfarming.client.geoviewer.LegendPalette;
import nl.cloudfarming.client.geoviewer.ValueRange;
import nl.cloudfarming.client.util.Gradient;
import nl.cloudfarming.client.util.UOM;

/**
 * Pallete for a {@link FieldTask}
 *
 * @author Merijn Zengers <Merijn.Zengers@gmail.nl>
 */
public class FieldTaskPalette extends LegendPalette<Integer> implements ValueRange<Double>, Serializable {

    public static final Color STEP_0 = new Color(0, 96, 255);
    public static final Color STEP_1 = new Color(0, 159, 255);
    public static final Color STEP_2 = new Color(0, 223, 255);
    public static final Color STEP_3 = new Color(32, 255, 223);
    public static final Color STEP_4 = new Color(96, 255, 159);
    public static final Color STEP_5 = new Color(159, 255, 96);
    public static final Color STEP_6 = new Color(223, 255, 32);
    public static final Color STEP_7 = new Color(255, 223, 0);
    public static final Color STEP_8 = new Color(255, 159, 0);
    public static final Color STEP_9 = new Color(255, 96, 0);
    private static final Color[] colors = new Color[]{STEP_0, STEP_1, STEP_2, STEP_3, STEP_4, STEP_5, STEP_6, STEP_7, STEP_8, STEP_9};
    private static final Gradient GRADIENT = Gradient.create(colors);
    private Double minValue;
    private Double maxValue;

    public FieldTaskPalette(Double minValue, Double maxValue, UOM uom) {
        super(STEP_0, uom);
        this.minValue = minValue;
        this.maxValue = maxValue;
    }

    @Override
    public Double getMinValue() {
        return minValue;
    }
    
    private double getMin(){
        return minValue==null?0.0d:minValue.doubleValue();
    }
    
    private double getMax(){
        return maxValue==null?0.0d:maxValue.doubleValue();
    }    

    public void setMinMaxValue(Double minValue, Double maxValue) {
        this.minValue = minValue;
        this.maxValue = maxValue;
        // Fire a single event on range updates; min and max can be changed 
        // simultaneously, e.g. when changing the value of a single valued task.
        // When doing separate updates (and firing separate events) the 
        // intermediate range can be invalid (min > max), causing errors in both
        // the legend and the grid coverage renderer.
        getPropertyChangeSupport().firePropertyChange(LegendPalette.PROP_LEGEND_COLORS, null, null);
    }

    @Override
    public Double getMaxValue() {
        return maxValue;
    }

    @Override
    public int getSteps() {
        return getMax() - getMin() > colors.length ? colors.length : Double.valueOf(getMax() - getMin()).intValue();
    }

    /**
     * Gets the color for the provided value
     *
     *
     * @param value the value for which the color should be returned
     * @return the color suited for the value
     */
    @Override
    public Color getColorForValue(Integer value) {
        double stepSize = getStepSize();

        for (int i = 0; i < getSteps(); i++) {
            if (value <= stepSize * i + getMin()) {
                return colors[i];
            }
        }
        return STEP_0;
    }
    
    // FIXME: use this as regular getColorForValue once the ColorMap in 
    // GridCoverage2DRenderer can handle doubles.
    public Color getGradientColorForValue(double value) {
        return GRADIENT.getColor(getMax()-getMin() != 0 ? (value-getMin())/(getMax()-getMin()) : 0.0);
    }

    private double getStepSize() {
        return getMax() - getMin() > 0 ? (getMax() - getMin()) / (getSteps() -1) : 0;
    }

    @Override
    public LegendPalette.LegendColors[] getColors() {
        double stepSize = getStepSize();
        int steps = getSteps();
        
        if (getMax() == getMin()) {
            return new FieldTaskLegendColors[] {new FieldTaskLegendColors(colors[0], Long.toString(Math.round(getMin())))};
        }
        
        FieldTaskLegendColors[] legendColors = new FieldTaskLegendColors[steps];

        double stepValue = getMin();
        for (int i = 0; i < steps; i++) {
            legendColors[i] = new FieldTaskLegendColors(colors[i], Long.toString(Math.round(stepValue)));
            stepValue += stepSize;
        }
        return legendColors;
    }

    private class FieldTaskLegendColors extends LegendPalette.LegendColors {

        private Color color;
        private String description;

        private FieldTaskLegendColors(Color color, String description) {
            this.color = color;
            this.description = description;
        }

        @Override
        public Color getColor() {
            return color;
        }

        @Override
        public String getDescription() {
            return description;
        }
    }
}
